package	jp.co.sds_corp.sqltool;

import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.sql.Types;

import jp.co.sds_corp.util.db.DbColumn;
import jp.co.sds_corp.util.db.DbTable;

public abstract class AbstractRowWriter implements IRowWriter
{
	ToolParam	prm = null;
	PrintWriter	pw = null;
	OutputStreamWriter	os = null;
	IFinisher	fin = null;
	int		nRow = 0;

	public AbstractRowWriter( ToolParam prm, PrintWriter pw ) {
		this.prm = prm;
		this.pw = pw;
		fin = new StreamFinisher( prm, pw );
	}

	public void setFinisher( IFinisher fin ) {
		this.fin = fin;
	}

	protected int getColumnLen( DbColumn col ) {
		int		nLen = col.getSize();
		switch ( col.getType() ) {
		case Types.DATE:
			if ( nLen < 26 ) {
				nLen = 26;
			}
			break;
		case Types.TIMESTAMP:
			if ( nLen < 26 ) {
				nLen = 26;
			}
			break;
		case Types.TIME:
			if ( nLen < 8 ) {
				nLen = 8;
			}
			break;
		case Types.LONGVARCHAR:
			if ( nLen > 80 ) {
				nLen = 80;
			}
			break;
		}
		if ( prm.getColLen() == ToolParam.COLLEN_NAME ) {
			if ( nLen < col.getName().length()) {
				nLen = col.getName().length();
			}
		}
		if ( nLen > 1024 ) {
			nLen = 1024;
		}
		return nLen;
	}

	public String getColumnString( DbTable tbl, int nCol ) {
		DbColumn col = tbl.getColumn( nCol );
		String ret;
		switch ( col.getType()) {
		case Types.DATE:
		case Types.TIMESTAMP:
			try {
				ret = tbl.formatDate( col.getName(), prm.getDateFormat() );
			} catch ( Exception e ) {
				ret = "";
			}
			break;
		case Types.TIME:
			try {
				ret = tbl.formatDate( col.getName(), prm.getTimeFormat() );
			} catch ( Exception e ) {
				ret = "";
			}
			break;
		case Types.FLOAT:
		case Types.DOUBLE:
			ret = tbl.formatDecimal( col.getName(), prm.getFloatFormat() );
			break;
		default:
			ret = tbl.getString( nCol );
			break;
		}
		if ( ret == null ) {
			ret = "";
		}
		return ret;
	}

	public void printMessage( String msg ) {
	}

	public void endPrint() {
		fin.endPrint();
	}
}
