package	jp.co.sds_corp.sqltool;

import java.io.PrintWriter;
import java.sql.Types;
import java.util.Arrays;

import jp.co.sds_corp.util.common.CmnStringBuffer;
import jp.co.sds_corp.util.db.DbColumn;
import jp.co.sds_corp.util.db.DbTable;

public class PrintRowWriter extends AbstractRowWriter
{
	public PrintRowWriter( ToolParam prm, PrintWriter pw ) {
		super( prm, pw );
	}

	protected int checkPos( int nPos, int nLen ) {
		if ( prm.getWinSize() == 0 ) {
			return nPos + nLen + 1;
		}
		if (( nPos + nLen + 1 ) > prm.getWinSize()) {
			pw.println( "" );
			nPos = nLen + 1;
		} else {
			nPos = nPos + nLen + 1;
		}
		return nPos;
	}

	protected int printColumnName( DbColumn col, int nPos, boolean bSep ) {
		int nLen = getColumnLen( col );
		nPos = checkPos( nPos, nLen );
		String str;
		if ( bSep ) {
			char[]  cSep = new char[ nLen ];
			Arrays.fill( cSep, '-' );
			str = new String( cSep );
		} else {
			CmnStringBuffer buf = new CmnStringBuffer( col.getName() );
			buf.setLength( nLen );
			str = buf.toString();
		}
		pw.print( str );
		return nPos;
	}

	public void printColumnInfo( DbTable tbl ) {
		int		nPos;
		DbColumn	col;

		nPos = 0;
		for ( int i = 0; i < tbl.getColumnCount(); i++ ) {
			col = tbl.getColumn( i );
			nPos = printColumnName( col, nPos, false );
			if ( i == tbl.getColumnCount() - 1 ) {
				pw.println( "" );
			} else {
				pw.print( " " );
			}
		}
		nPos = 0;
		for ( int i = 0; i < tbl.getColumnCount(); i++ ) {
			col = tbl.getColumn( i );
			nPos = printColumnName( col, nPos, true );
			if ( i == tbl.getColumnCount() - 1 ) {
				pw.println( "" );
			} else {
				pw.print( " " );
			}
		}
		nRow = 0;
	}

	public void printRow( DbTable tbl ) {
		int		nPos = 0;
		for ( int i = 0; i < tbl.getColumnCount(); i++ ) {
			DbColumn col = tbl.getColumn( i );
			int		nLen = getColumnLen( col );
			nPos = checkPos( nPos, nLen );
			//String data = tbl.getString( i );
			String data = getColumnString( tbl, i );
			CmnStringBuffer buf = new CmnStringBuffer( data );
			switch ( col.getType() ) {
			case Types.TINYINT:
			case Types.BIGINT:
			case Types.INTEGER:
			case Types.SMALLINT:
			case Types.NUMERIC:
			case Types.DECIMAL:
			case Types.FLOAT:
			case Types.DOUBLE:
				buf.setLength( nLen, false );
				break;
			case Types.BINARY:
				break;
			default:
				buf.setLength( nLen );
				break;
			}
			pw.print( buf.toString() );
			if ( i == tbl.getColumnCount() - 1 ) {
				pw.println( "" );
			} else {
				pw.print( " " );
			}
		}
		nRow++;
	}

	public void printMessage( String msg ) {
		pw.println( msg );
	}

	public void endPrint() {
		fin.setRowCount( nRow );
		super.endPrint();
	}
}
