package	jp.co.sds_corp.sqltool;

import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.SQLWarning;

import jp.co.sds_corp.util.common.CmnString;
import jp.co.sds_corp.util.db.DbAccess;
import jp.co.sds_corp.util.db.DbColumn;
import jp.co.sds_corp.util.db.DbException;
import jp.co.sds_corp.util.db.DbTable;

public class SqlEngine
{
	DbAccess	db;
	ToolParam	prm;

	public SqlEngine( DbAccess db, ToolParam prm ) throws Exception {
		this.db = db;
		this.prm = prm;
	}

	public int fetch( DbAccess db, IRowWriter writer ) throws DbException {
		DbTable	tbl = new DbTable();
		int		nRow;
		boolean bRet;
		nRow = 0;
		for ( ;; ) {
			bRet = db.fetch( tbl );
			if ( nRow == 0 ) {
				writer.printColumnInfo( tbl );
			}
			if ( !bRet ) {
				break;
			}
			nRow++;

			writer.printRow( tbl );
		}
		return nRow;
	}

	protected void printWarnings( IRowWriter writer ) throws DbException {
		SQLWarning sw = db.getWarnings();
		for ( ;; ) {
			if ( sw == null ) {
				break;
			}
			writer.printMessage( sw.getMessage() );
			sw = sw.getNextWarning();
		}
		db.clearWarnings();
	}

	protected boolean select( String sql, IRowWriter writer ) {
		try {
			int		nRow = -1;
			int		nUpd = -1;

			if ( sql.length() == 0 ) {
				return false;
			}

			boolean bRet = db.execute( sql );
			for ( ;; ) {
				if ( nRow >= 0 || nUpd >= 0 ) {
					bRet = db.getMoreResults();
				}
				printWarnings( writer );
				if ( bRet ) {
					nRow = fetch( db, writer );
					writer.printMessage( "(" + nRow + " rows affected)" );
				} else {
					nUpd = db.getUpdateCount();
					if ( nUpd > 0 ) {
						writer.printMessage( "(" + nUpd + " rows updated)" );
					}
				}
				if ( !bRet && nUpd < 0 ) {
					break;
				}
			}
			printWarnings( writer );
			writer.endPrint();
/*
			if ( prm.isAutoCommit() ) {
				db.commit();
			}
*/
			db.cancel();
			return true;
		} catch ( DbException dbe ) {
			if ( prm.isDebug()) {
				dbe.printStackTrace();
			} else {
				if ( dbe.getCause() == null ) {
					System.err.println( dbe.getMessage());
				} else {
					if ( dbe.getCause() instanceof SQLException ) {
						SQLException se = (SQLException )dbe.getCause();
						System.err.println( se.getSQLState() + ":" + se.getMessage());
					} else {
						System.err.println( dbe.getCause() );
					}
				}
			}
		} catch ( Exception ex ) {
			if ( prm.isDebug()) {
				ex.printStackTrace();
			} else {
				System.err.println( ex.getClass().getName() + ":" + ex.getMessage());
			}
		} catch ( Throwable th ) {
			th.printStackTrace();
		} finally {
			try {
				db.cancel();
			} catch ( Exception ex ) {
			}
		}
		return true;
	}

	protected void describe( String table, IRowWriter writer, boolean bAll ) {
		try {
			String catalog = null;
			String schema = null;
			String[] arr = CmnString.getStringToken( table, "." );
			if ( arr.length == 2 ) {
				schema = arr[0];
				table = arr[1];
			} else if ( arr.length == 3 ) {
				catalog = arr[0];
				schema = arr[1];
				table = arr[2];
			}
			DatabaseMetaData meta = db.getConnection().getMetaData();
			ResultSet rset = meta.getColumns( catalog, schema, table, null );
			db.setResultSet( rset );

			DbTable desc = new DbTable();
			DbTable	tbl = new DbTable();
			int		nRow = 0;
			boolean bRet;
			for ( ;; ) {
				bRet = db.fetch( tbl );
				if ( nRow == 0 ) {
					if ( !bRet ) {
						System.err.println( "table [" + table + "] not found." );
						break;
					}
					copyColumn( desc, tbl );
					if ( bAll ) {
						writer.printColumnInfo( tbl );
					} else {
						writer.printColumnInfo( desc );
					}
				}
				if ( !bRet ) {
					break;
				}
				nRow++;
				if ( bAll ) {
					writer.printRow( tbl );
				} else {
					writer.printRow( desc );
				}
			}
			writer.endPrint();
		} catch ( Exception ex ) {
			if ( prm.isDebug()) {
				ex.printStackTrace();
			} else {
				System.err.println( ex.getClass().getName() + ":" + ex.getMessage());
			}
		} finally {
			try {
				db.cancel();
			} catch ( Exception ex ) {
			}
		}
	}

	String[] cols = {
		"COLUMN_NAME", "TYPE_NAME", "COLUMN_SIZE", "DECIMAL_DIGITS", "NULLABLE"
	};
	private void copyColumn( DbTable dst, DbTable src ) {
		for ( int i = 0; i < cols.length; i++ ) {
			DbColumn col = src.getColumn( cols[i] );
			dst.addColumn( col );
		}
	}

}

