package	jp.co.sds_corp.sqltool;

import java.io.BufferedReader;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;

import jp.co.sds_corp.util.common.CmnManifest;
import jp.co.sds_corp.util.common.CmnString;
import jp.co.sds_corp.util.db.DbAccess;
import jp.co.sds_corp.util.db.DbException;
import jp.co.sds_corp.util.db.DbInfo;

public class SqlTool
{
	static String mankey = "jp/co/sds_corp/sqltool/";
	CmnManifest mn = new CmnManifest( getClass());
	String title = mn.getTitle( mankey );
	String version = mn.getVersion( mankey );

	DbAccess	db = null;
	SqlEngine	engine = null;
	SqlHistory hist = null;
	ToolParam prm = null;
	IRowWriter defwriter = null;
	RowWriterFactory rwFactory = null;
	SqlEditor editor = null;
	FileLoader loader = null;

	public SqlTool() throws Exception {
		Thread shut = new Thread() {
			public void run() {
				if ( hist != null ) {
					hist.write();
				}
			}
		};
		Runtime.getRuntime().addShutdownHook( shut );

		prm = new ToolParam();
		DbInfo info = prm.getDbInfo();
		db = new DbAccess( info );
		db.connect();
		db.setAutoCommit( prm.isAutoCommit() );
		db.setTrim( prm.isTrim() );

		engine = new SqlEngine( db, prm );
		hist = new SqlHistory( prm );
		rwFactory = new RowWriterFactory( prm );
		defwriter = rwFactory.createStdoutWriter();
		editor = new SqlEditor( prm, hist );
		loader = new FileLoader( prm, db );
	}

	private void showHelp() {
		System.out.println( "help\t\t: show this help" );
		System.out.println( "reset\t\t: cancel sql statement." );
		System.out.println( "quit\t\t: quit this program." );
		System.out.println( "history\t\t: show sql history" );
		System.out.println( "vi\t\t: edit previous sql." );
		System.out.println( "vi ![#]\t\t: edit sql. refer event no #" );
		System.out.println( "vi ![str]\t: edit sql. search history starts with str" );
		System.out.println( "![#]\t\t: search history. refer event no #." );
		System.out.println( "![str]\t\t: search history. starts with str." );
		System.out.println( "go\t\t: execute sql statement." );
		System.out.println( "gom\t\t: execute sql statement. display results with more" );
		System.out.println( "gov\t\t: execute sql statement. display results vertically" );
		System.out.println( "got [filename]\t: execute sql statement. write result to file" );
		System.out.println( "goc [filename]\t: execute sql statement. write result to CSV file" );
		System.out.println( "desc [table]\t: describe table. results are stored as result set." );
		System.out.println( "descall [table]\t: describe table. results are stored as result set." );
		System.out.println( ":r [filename]\t: read sql statement from file." );
		System.out.println( ":w [filename]\t: write current sql statement to file." );
		System.out.println( ":load [table] [filename]\t: load file to table." );
		System.out.println( ":save [table] [filename]\t: save table to file." );
		System.out.println( ":set\t\t: show :set usage." );
		System.out.println( ":set all\t: show all values." );
		System.out.println( ":set var=value\t: set value to var." );
		System.out.println( ":ver\t\t: show version" );
	}

	private String getFileName( String buf ) {
		String[] vars = CmnString.getStringToken( buf, " " );
		if ( vars.length < 2 ) {
			return null;
		}
		return vars[1];
	}

	private String readSql( String buf ) {
		String file = getFileName( buf );
		if ( file == null ) {
			System.out.println( "file not found:[" + buf + "]" );
			return null;
		}
		String sql = null;
		BufferedReader in = null;
		try {
			in = new BufferedReader( new FileReader( file ));
			sql = "";
			for ( ;; ) {
				String line = in.readLine();
				if ( line == null ) {
					break;
				}
				sql = sql.concat( line + " " );
			}
		} catch ( Exception ex ) {
			System.out.println( "can't read file :[" + file + "]" );
			return null;
		} finally {
			if ( in != null ) {
				try {
					in.close();
				} catch ( IOException ex ) {
					// ignore
				}
			}
		}
		System.out.println( "read SQL file :[" + file + "]" );
		return sql;
	}

	private void writeSql( String buf ) {
		String file = getFileName( buf );
		if ( file == null ) {
			System.out.println( "file not found:[" + buf + "]" );
			return;
		}
		String sql = hist.search( "!!" );
		if ( sql == null ) {
			System.out.println( "SQL not found" );
			return;
		}
		try {
			PrintWriter pw = new PrintWriter( new FileOutputStream( file ));
			pw.println( sql );
			pw.flush();
			pw.close();
		} catch ( IOException ex ) {
			System.out.println( "can't write file :[" + file + "]" );
			return;
		}
		System.out.println( "write SQL file :[" + file + "]" );
	}

	private void loadFile( String buf ) {
		String[] vars = CmnString.getStringToken( buf, " " );
		if ( vars.length != 3 ) {
			System.out.println( "must be :load [table] [file]:[" + buf + "]" );
			return;
		}
		try {
			loader.loadFile( vars[1], vars[2] );
		} catch ( DbException dbe ) {
			System.err.println( dbe.getCause().getMessage());
		} catch ( Exception ex ) {
			System.err.println( ex.getMessage());
		} catch ( Throwable th ) {
			th.printStackTrace();
		}
	}

	private void saveFile( String buf ) {
		String[] vars = CmnString.getStringToken( buf, " " );
		if ( vars.length != 3 ) {
			System.out.println( "must be :save [table] [file]:[" + buf + "]" );
			return;
		}
		try {
			boolean bFlg = prm.isWithCol();
			prm.setWithCol( "off" );
			String sql = "select * from " + vars[1];
			IRowWriter writer = rwFactory.createCsvWriter( vars[2] );
			engine.select( sql, writer );
			prm.setWithCol( bFlg ? "on" : "off" );
		} catch ( Exception ex ) {
			System.err.println( ex.getMessage());
		} catch ( Throwable th ) {
			th.printStackTrace();
		}
	}

	private void cmd() {
		try {
			int nHist = 0;
			int nPrompt = 0;
			String sql = "";
			IRowWriter writer = null;

			BufferedReader in = new BufferedReader(
								new InputStreamReader( System.in ));
			for ( ;; ) {
				nPrompt++;
				System.out.print( nPrompt + "> " );
				String buf = in.readLine();
				if ( buf == null ) {
					System.out.println( "" );
					break;
				}
				buf = buf.trim();
				if ( buf.length() == 0 ) {
					continue;
				}
				if ( buf.startsWith( "#" )) {
					nPrompt--;
					continue;
				} else if ( buf.startsWith( "!" )) {
					String tmp = hist.search( buf );
					if ( tmp == null ) {
						System.out.println( buf + ": no such event" );
						continue;
					} else {
						nPrompt = 1;
						System.out.println( nPrompt + "> " + tmp );
						if ( tmp.startsWith( ":" )) {
							buf = tmp;
						} else {
							sql = tmp;
							continue;
						}
					}
				}

				writer = defwriter;
				String file;
				if ( buf.startsWith( "got " )) {
					file = getFileName( buf );
					if ( file == null ) {
						System.out.println( "file not found:[" + buf + "]" );
						continue;
					}
					writer = rwFactory.createPrintWriter( file );
					if ( writer == null ) {
						nPrompt--;
						continue;
					}
					buf = "go";
				} else if ( buf.startsWith( "goc " )) {
					file = getFileName( buf );
					if ( file == null ) {
						writer = rwFactory.createCsvWriter( System.out );
					} else {
						writer = rwFactory.createCsvWriter( file );
					}
					if ( writer == null ) {
						nPrompt--;
						continue;
					}
					buf = "go";
				} else if ( buf.equals( "gom" )) {
					writer = rwFactory.createMoreWriter();
					if ( writer == null ) {
						nPrompt--;
						continue;
					}
					buf = "go";
				} else if ( buf.equals( "gov" )) {
					writer = rwFactory.createVerticalWriter();
					if ( writer == null ) {
						nPrompt--;
						continue;
					}
					buf = "go";
				}

				if ( buf.equals( "quit" )) {
					break;
				} else if ( buf.equals( "go" )) {
					if ( sql.startsWith( "desc " )) {
						String tbl = getFileName( sql );
						engine.describe( tbl, writer, false );
					} else
					if ( sql.startsWith( "descall " )) {
						String tbl = getFileName( sql );
						engine.describe( tbl, writer, true );
					} else {
						boolean bRet = engine.select( sql, writer );
						if ( !bRet ) {
							sql = "";
							nPrompt = 0;
						}
					}
					nHist++;
					nPrompt = 0;
					hist.add( sql );
					sql = "";
				} else if ( buf.equals( "reset" )) {
					sql = "";
					nPrompt = 0;
				} else if ( buf.startsWith( "vi" )) {
					sql = editor.edit( buf, sql );
					if ( sql == null ) {
						sql = "";
						nPrompt--;
					} else {
						nPrompt = 1;
						System.out.println( "1> " + sql );
					}
				} else if ( buf.equals( "help" )) {
					showHelp();
					nPrompt--;
				} else if ( buf.equals( "history" )) {
					hist.showAll();
					nPrompt--;
				} else if ( buf.startsWith( ":" )) {
					if ( buf.startsWith( ":set" )) {
						boolean bAuto = prm.isAutoCommit();
						boolean bTrim = prm.isTrim();
						prm.setVar( buf );
						if ( bAuto != prm.isAutoCommit()) {
							db.setAutoCommit( prm.isAutoCommit() );
						}
						if ( bTrim != prm.isTrim()) {
							db.setTrim( prm.isTrim() );
						}
						defwriter = rwFactory.createStdoutWriter();
						nPrompt--;
					} else if ( buf.startsWith( ":r" )) {
						sql = readSql( buf );
						nPrompt = 1;
						if ( sql != null ) {
							System.out.println( "1> " + sql );
						}
					} else if ( buf.startsWith( ":w" )) {
						writeSql( buf );
						nPrompt--;
					} else if ( buf.equals( ":ver" )) {
						System.out.println( title + " ver " + version );
						nPrompt--;
					} else if ( buf.startsWith( ":load" )) {
						loadFile( buf );
						nPrompt = 0;
						hist.add( buf );
						sql = "";
					} else if ( buf.startsWith( ":save" )) {
						saveFile( buf );
						nPrompt = 0;
						hist.add( buf );
						sql = "";
					} else {
						showHelp();
						nPrompt--;
					}
				} else {
					sql = sql.concat( buf );
					sql = sql.concat( " " );
				}
			}
		} catch ( Throwable ex ) {
			System.out.println( "Exception=" + ex.getMessage());
			ex.printStackTrace();
		}
		try {
			db.disconnect();
		} catch ( Throwable ex ) {
		}
	}

	public static void main( String[] args ) {
		try {
/*
			String var = System.getProperty( "sql.logconfig" );
			if ( var != null ) {
				CmnLog.configure( var );
			}
*/
			if ( args.length > 0 && args[0].equals( "--version" )) {
				CmnManifest mn = new CmnManifest( SqlTool.class );
				System.out.println( mn.getTitle( mankey )
						+ " ver " + mn.getVersion( mankey ));
			} else {
				SqlTool	obj = new SqlTool();
				obj.cmd();
			}
		} catch ( Exception ex ) {
			ex.printStackTrace();
		}
	}
}

