package	jp.co.sds_corp.sqltool;

import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.util.Iterator;
import java.util.PropertyResourceBundle;
import java.util.ResourceBundle;

import jp.co.sds_corp.util.common.CmnProperty;
import jp.co.sds_corp.util.common.CmnString;
import jp.co.sds_corp.util.config.DOMConfiguration;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.db.DbException;
import jp.co.sds_corp.util.db.DbInfo;

public class ToolParam
{
	static String PROP_DRIVER		= "driver";
	static String PROP_ENCODING		= "encoding";
	static String PROP_EDITOR		= "editor";
	static String PROP_LESS			= "less";
	static String PROP_HIST_FILE	= "historyfile";
	static String PROP_AUTO_COMMIT	= "autocommit";
	static String PROP_TRIM			= "trim";
	static String PROP_WIN_SIZE		= "winsize";
	static String PROP_MAX_HIST		= "history";
	static String PROP_SAVE_HIST	= "savehist";
	static String PROP_MODE			= "mode";
	static String PROP_WITH_COL		= "withcol";
	static String PROP_COL_LEN		= "collen";
	static String PROP_COL_SEP		= "colsep";
	static String PROP_ROW_SEP		= "rowsep";
	static String PROP_FLOAT_FMT	= "floatfmt";
	static String PROP_DATE_FMT		= "datefmt";
	static String PROP_TIME_FMT		= "timefmt";

	DbInfo info = null;
	CmnProperty prm;
	String driver = null;
	String encoding = null;
	String editor = null;
	String less = null;
	String historyFile = null;
	boolean bAutoCommit = false;
	boolean trim = false;
	int		nMaxHist = 100;
	boolean	bSaveHist = true;
	int		nWinSize = 80;
	int		nMode = 0;
	static int MODE_HORZ = 0;
	static int MODE_VERT = 1;

	boolean	bWithCol = true;
	int		nColLen = 0;
	static int COLLEN_NAME = 0;
	static int COLLEN_DATA = 1;

	char	cColSep = '\t';
	char	cRowSep = '\n';
	boolean	debug = false;
	int		debugLevel = 0;

	String	fltfmt = "0.000000000";
	String	dtfmt  = "yyyy-MM-dd HH:mm:ss";
	String	tmfmt  = "HH:mm:ss";

	public ToolParam() throws Exception {
		prm = new CmnProperty( "sqltool" );

		String buf;
		try {
			buf = prm.getString( PROP_DRIVER, null );
			if ( buf != null ) {
				setDriver( buf );
			}
			buf = prm.getString( PROP_ENCODING, null );
			if ( buf != null ) {
				setEncoding( buf );
			}
			buf = prm.getString( PROP_EDITOR, null );
			if ( buf == null ) {
				buf = "vi";
			}
			setEditor( buf );
			buf = prm.getString( PROP_LESS, null );
			if ( buf == null ) {
				buf = "more";
			}
			setLess( buf );
			buf = prm.getString( PROP_HIST_FILE, null );
			if ( buf == null ) {
				buf = System.getProperty( "user.home" );
				buf = buf.concat( System.getProperty( "file.separator" ));
				buf = buf.concat( ".sqlhist" );
			}
			setHistoryFile( buf );
			buf = prm.getString( PROP_AUTO_COMMIT, null );
			if ( buf == null ) {
				buf = "off";
			}
			setAutoCommit( buf );
			buf = prm.getString( PROP_TRIM, null );
			if ( buf == null ) {
				buf = "off";
			}
			setTrim( buf );
			buf = prm.getString( PROP_WIN_SIZE, null );
			if ( buf == null ) {
				buf = "80";
			}
			setWinSize( buf );
			buf = prm.getString( PROP_MAX_HIST, null );
			if ( buf == null ) {
				buf = "200";
			}
			setMaxHist( buf );
			buf = prm.getString( PROP_SAVE_HIST, null );
			if ( buf == null ) {
				buf = "on";
			}
			setSaveHist( buf );
			buf = prm.getString( PROP_MODE, null );
			if ( buf == null ) {
				buf = "horz";
			}
			setMode( buf );
			buf = prm.getString( PROP_WITH_COL, null );
			if ( buf == null ) {
				buf = "on";
			}
			setWithCol( buf );
			buf = prm.getString( PROP_COL_LEN, null );
			if ( buf == null ) {
				buf = "name";
			}
			setColLen( buf );
			buf = prm.getString( PROP_COL_SEP, null );
			if ( buf == null ) {
				buf = "\t";
			}
			setColSep( buf );
			buf = prm.getString( PROP_ROW_SEP, null );
			if ( buf == null ) {
				buf = "\n";
			}
			setRowSep( buf );
			buf = prm.getString( PROP_FLOAT_FMT, null );
			if ( buf == null ) {
				buf = "0.000000000";
			}
			setFloatFormat( buf );
			buf = prm.getString( PROP_DATE_FMT, null );
			if ( buf == null ) {
				buf = "yyyy-MM-dd HH:mm:ss";
			}
			setDateFormat( buf );
		} catch ( ParamException pe ) {
			showUsage();
		}

		// コマンドラインオプションから
		buf = System.getProperty( "sql.driver" );
		if ( buf != null ) {
			setDriver( buf );
		}
		buf = System.getProperty( "sql.encoding" );
		if ( buf != null ) {
			setEncoding( buf );
		}
		buf = System.getProperty( "sql.editor" );
		if ( buf != null ) {
			setEditor( buf );
		}
		buf = System.getProperty( "sql.less" );
		if ( buf != null ) {
			setLess( buf );
		}
		buf = System.getProperty( "sql.debug" );
		if ( buf != null ) {
			setDebug( true );
			setDebugLevel( buf );
		}

		buf = System.getProperty( "sql.config" );
		if ( buf != null ) {
			DOMConfiguration root = new DOMConfiguration( buf );
			Iterator<IConfiguration> itr = root.getChilds( "db" );
			while ( itr.hasNext()) {
				IConfiguration config = (IConfiguration )itr.next();
				String name = config.getAttributes().get( "name" );
				if ( getDriver() == null ) {
					info = new DbInfo( config );
					break;
				}
				if ( getDriver().equals( name )) {
					info = new DbInfo( config );
					break;
				}
			}
			if ( info == null ) {
				throw( new DbException( "name[" + getDriver() + "] not found." ));
			}
		} else {
			ResourceBundle prop = PropertyResourceBundle.getBundle( "DbEnv" );
			if ( getDriver() == null ) {
				info = new DbInfo( prop );
			} else {
				info = new DbInfo( prop, getDriver() );
			}
		}
	}

	public DbInfo getDbInfo() {
		return info;
	}

	public void setVar( String buf ) {
		if ( buf.length() < 6 ) {
			showUsage();
			return;
		}

		String[] vars = CmnString.getStringToken( buf.substring( 5 ), "=" );
		if (( vars.length == 1 ) && ( vars[0].equals( "all" ))) {
			showSet();
			return;
		}
		if ( vars.length != 2 ) {
			showUsage();
			return;
		}

		try {
			String key = vars[0];
			String val = vars[1];
			if ( key.equals( "editor" ) ) {
				setEditor( val );
			} else if ( key.equals( PROP_LESS ) ) {
				setLess( val );
/*
			} else if ( key.equals( PROP_DRIVER ) ) {
				setDriver( val );
*/
			} else if ( key.equals( PROP_ENCODING ) ) {
				setEncoding( val );
			} else if ( key.equals( PROP_AUTO_COMMIT ) ) {
				setAutoCommit( val );
			} else if ( key.equals( PROP_TRIM ) ) {
				setTrim( val );
			} else if ( key.equals( PROP_WIN_SIZE ) ) {
				setWinSize( val );
			} else if ( key.equals( PROP_MAX_HIST ) ) {
				setMaxHist( val );
			} else if ( key.equals( PROP_SAVE_HIST ) ) {
				setSaveHist( val );
			} else if ( key.equals( PROP_MODE ) ) {
				setMode( val );
			} else if ( key.equals( PROP_WITH_COL ) ) {
				setWithCol( val );
			} else if ( key.equals( PROP_COL_LEN ) ) {
				setColLen( val );
			} else if ( key.equals( PROP_COL_SEP ) ) {
				setColSep( val );
			} else if ( key.equals( PROP_ROW_SEP ) ) {
				setRowSep( val );
			} else if ( key.equals( PROP_FLOAT_FMT ) ) {
				setFloatFormat( val );
			} else if ( key.equals( PROP_DATE_FMT ) ) {
				setDateFormat( val );
			} else if ( key.equals( PROP_TIME_FMT ) ) {
				setTimeFormat( val );
			} else {
				showUsage();
				return;
			}
		} catch ( Exception ex ) {
			showUsage();
		}
	}

	boolean getOnOff( String val ) throws ParamException {
		boolean bRet = false;
		if ( val.equals( "on" ) ) {
			bRet = true;
		} else if ( val.equals( "off" ) ) {
			bRet = false;
		} else {
			throw( new ParamException( "[on] or [off] allowed" ));
		}
		return bRet;
	}

	void setDriver( String val ) {
		driver = val;
	}
	public String getDriver() {
		return driver;
	}

	void setEncoding( String val ) {
		try {
			OutputStreamWriter ow = new OutputStreamWriter( System.out, val );
			encoding = val;
		} catch ( UnsupportedEncodingException ue ) {
			System.out.println( "Illegal Encoding [" + val + "]" );
		}
	}
	public String getEncoding() {
		return encoding;
	}

	void setEditor( String val ) {
		editor = val;
	}
	public String getEditor() {
		return editor;
	}

	void setLess( String val ) {
		less = val;
	}
	public String getLess() {
		return less;
	}

	void setAutoCommit( String val ) {
		bAutoCommit = getOnOff( val );
	}
	public boolean isAutoCommit() {
		return bAutoCommit;
	}

	void setTrim( String val ) {
		trim = getOnOff( val );
	}
	public boolean isTrim() {
		return trim;
	}

	void setWinSize( String val ) {
		int nVal = Integer.valueOf( val ).intValue();
		nWinSize = nVal;
	}
	public int getWinSize() {
		return nWinSize;
	}

	public void setHistoryFile( String val ) {
		historyFile = val;
	}
	public String getHistoryFile() {
		return historyFile;
	}

	void setMaxHist( String val ) {
		int nVal = Integer.valueOf( val ).intValue();
		if (( nVal < 1 ) || ( nVal > 1000 )) {
			throw( new ParamException( "illegal history number:" + val ));
		}
		nMaxHist = nVal;
	}
	public int getMaxHist() {
		return nMaxHist;
	}

	void setSaveHist( String val ) {
		bSaveHist = getOnOff( val );
	}
	public boolean isSaveHist() {
		return bSaveHist;
	}

	void setMode( String val ) {
		if ( val.equals( "vert" ) ) {
			nMode = MODE_VERT;
		} else if ( val.equals( "horz" ) ) {
			nMode = MODE_HORZ;
		} else {
			throw( new ParamException( "illegal value :" + val ));
		}
	}
	public int getMode() {
		return nMode;
	}

	void setWithCol( String val ) {
		bWithCol = getOnOff( val );
	}
	public boolean isWithCol() {
		return bWithCol;
	}

	void setColLen( String val ) {
		if ( val.equals( "name" ) ) {
			nColLen = COLLEN_NAME;
		} else if ( val.equals( "data" ) ) {
			nColLen = COLLEN_DATA;
		} else {
			throw( new ParamException( "illegal value :" + val ));
		}
	}
	public int getColLen() {
		return nColLen;
	}

	void setColSep( String val ) {
		if ( val.charAt( 0 ) == '\\' ) {
			switch ( val.charAt( 1 ) ) {
//			case 'a': cColSep = '\a'; break;
			case 'f': cColSep = '\f'; break;
			case 'n': cColSep = '\n'; break;
			case 't': cColSep = '\t'; break;
//			case 'v': cColSep = '\v'; break;
			case '\\': cColSep = '\\'; break;
			default:
				throw( new ParamException( "illegal value :" + val ));
			}
		} else {
			cColSep = val.charAt( 0 );
		}
	}
	public char getColSep() {
		return cColSep;
	}

	void setRowSep( String val ) {
		if ( val.charAt( 0 ) == '\\' ) {
			switch ( val.charAt( 1 ) ) {
//			case 'a': cRowSep = '\a'; break;
			case 'f': cRowSep = '\f'; break;
			case 'n': cRowSep = '\n'; break;
			case 't': cRowSep = '\t'; break;
//			case 'v': cRowSep = '\v'; break;
			case '\\': cRowSep = '\\'; break;
			default:
				throw( new ParamException( "illegal value :" + val ));
				}
		} else {
			cRowSep = val.charAt( 0 );
		}
	}
	public char getRowSep() {
		return cRowSep;
	}

	void setFloatFormat( String val ) {
		fltfmt = val;
	}
	public String getFloatFormat() {
		return fltfmt;
	}

	void setDateFormat( String val ) {
		dtfmt = val;
	}
	public String getDateFormat() {
		return dtfmt;
	}

	void setTimeFormat( String val ) {
		tmfmt = val;
	}
	public String getTimeFormat() {
		return tmfmt;
	}

	public boolean isDebug() {
		return debug;
	}
	public void setDebug( boolean var ) {
		debug = var;
	}

	public int getDebugLevel() {
		return debugLevel;
	}
	public void setDebugLevel( String var ) {
		try {
			debugLevel = Integer.valueOf( var ).intValue();
		} catch ( Exception ex ) {
		}
	}

	private String printChar( char cChr ) {
		switch ( cChr ) {
//		case '\a': return "\\a";
		case '\f': return "\\f";
		case '\n': return "\\n";
		case '\t': return "\\t";
//		case '\v': return "\\v";
		default  : return String.valueOf( cChr );
		}
	}

	private void showSet()
	{
		String key = getDriver();
		if ( key == null ) {
			key = "<None>";
		}
		System.out.println( "key     : " + key );
		System.out.println( "driver  : " + info.getDriver() );
		System.out.println( "url     : " + info.getUrl() );
		System.out.println( "user    : " + info.getUser() );
		String enc = info.getEncode();
		if ( enc == null ) {
			enc = "<Default>";
		}
		System.out.println( "encode  : " + enc );
		System.out.println( "history : " + historyFile );
		System.out.println( "" );
		System.out.println( "var\t\tvalue" );
		System.out.println( "--------------------------" );
		enc = getEncoding();
		if ( enc == null ) {
			enc = "<Default>";
		}
		System.out.println( "encoding\t" + enc );
		System.out.println( "editor\t\t" + editor );
		System.out.println( "less\t\t" + less );
		System.out.println( "history\t\t" + nMaxHist );
		System.out.println( "savehist\t"
						+ ( bSaveHist ? "on" : "off" ));
		System.out.println( "autocommit\t"
						+ ( bAutoCommit ? "on" : "off" ));
		System.out.println( "trim\t\t"
						+ ( trim ? "on" : "off" ));
		System.out.println( "winsize\t\t" + nWinSize );
		System.out.println( "mode\t\t"
						+ ( nMode == MODE_VERT ? "vertical" : "horizontal" ));
		System.out.println( "collen\t\t"
						+ ( nColLen == COLLEN_DATA ? "data" : "name" ));
		System.out.println( "withcol\t\t"
						+ ( bWithCol ? "on" : "off" ));
		System.out.println( "colsep\t\t" + printChar( cColSep ));
		System.out.println( "rowsep\t\t" + printChar( cRowSep ));
		System.out.println( "floatfmt\t" + fltfmt );
		System.out.println( "datefmt\t\t" + dtfmt );
		System.out.println( "timefmt\t\t" + tmfmt );
	}

	private void showUsage()
	{
		System.out.println( ":set all" );
		System.out.println( ":set var=value" );
		System.out.println( "var\t\tvalue" );
		System.out.println( "------------------------" );
/*
		System.out.println( "driver\t\tDriver" );
*/
		System.out.println( "encoding\tEncoding" );
		System.out.println( "editor\t\tSQL editor" );
		System.out.println( "less\t\tResult viewer with gom command." );
		System.out.println( "history\t\thistory size (1-1000)" );
		System.out.println( "savehist\tsave history (on/off)" );
		System.out.println( "autocommit\tauto commit SQL (on/off)" );
		System.out.println( "trim\t\ttrim string (on/off)" );
		System.out.println( "winsize\t\twindow size" );
		System.out.println( "mode\t\tshow mode (horz/vert)" );
		System.out.println( "collen\t\tcolumn length (name/data)" );
		System.out.println( "withcol\t\twith column name to CSV file (on/off)" );
		System.out.println( "colsep\t\tColumn separator on CSV file" );
		System.out.println( "rowsep\t\tRow Separator on CSV file" );
		System.out.println( "floatfmt\tFLOAT print format. Refer to the printf()" );
		System.out.println( "datefmt\t\tDATE conversion format. Refer to class DateFormat" );
		System.out.println( "timefmt\t\tTIME conversion format. Refer to class DateFormat" );
	}

}
